(function () {
  var appList = ['app.bundle.js', 'styles.bundle.js']
  var preList = ['lib.bundle.js']
  var wholeList = ['lib.bundle.js', 'app.bundle.js', 'styles.bundle.js']
  var d = new Date()
  var t = d.getTime()
  d = null

  // 为了和app业务逻辑代码一致
  var ls = {
    setItem: function (key, value) {
      key = JSON.stringify(key)
      value = JSON.stringify(value)
      return localStorage.setItem(key, value)
    },
    getItem: function (key) {
      let result = null
      key = JSON.stringify(key)
      try { result = JSON.parse(localStorage.getItem(key)) } catch (e) { result = null }
      return result
    },
    removeItem: function (key) {
      key = JSON.stringify(key)
      return localStorage.removeItem(key)
    },
    clear: function () {
      localStorage.clear()
    }
  }

  // 为了和app业务逻辑代码一致
  var ss = {
    setItem: function (key, value) {
      key = JSON.stringify(key)
      value = JSON.stringify(value)
      return sessionStorage.setItem(key, value)
    },
    getItem: function (key) {
      let result = null
      key = JSON.stringify(key)
      try { result = JSON.parse(sessionStorage.getItem(key)) } catch (e) { result = null }
      return result
    },
    removeItem: function (key) {
      key = JSON.stringify(key)
      return sessionStorage.removeItem(key)
    },
    clear: function () {
      sessionStorage.clear()
    }
  }

  var LocalStatus = ls.getItem('hotfix-store-status')
  var LocalNative = ls.getItem('hotfix-store-native')
  var SessStack = ss.getItem('hotfix-resume-stack')
  var SessPath = ss.getItem('hotfix-resume-path')
  /*
  * @
  * @ 热更新loader
  * @
  **/
  var loader = {
    /*
     * @
     * @ 拉取javascript函数(非阻塞)
     * @
     **/
    loadJavascript: function (url, callback) {
      var node = document.createElement('script')
      var that = this
      var err = function () { that.clearAll(); location.hash = ''; location.reload() }
      var loaded = function () { node && node.readyState && node.readyState != 'loaded' && node.readyState != 'complete' || (node.onload = node.onreadystatechange = node.onerror = null, node.src = '', node.parentNode.removeChild(node), node = null, callback && typeof callback === 'function' && callback()) }
      node.style.display = 'none'
      node.src = url
      node.onload = node.onreadystatechange = loaded, document.getElementsByTagName('body')[0].appendChild(node)
      node.onerror = err
    },
    /*
     * @
     * @ 拉取javascript函数(阻塞)
     * @
     **/
    syncLoadJavascript: function (path, scripts) {
      var that = this
      var cb = function () { that.syncLoadJavascript(path, scripts) }
      var js
      if (scripts && scripts.length > 0) {
        js = scripts.shift()
        if (js === 'cordova.js') {
          this.loadJavascript('./cordova.js', cb)
        } else {
          this.loadJavascript(path + js + '?t=' + t, cb)
        }
      }
    },
    /*
     * @
     * @ 清除业务ls
     * @
     **/
    clearAppLS: function () {
      ls.removeItem('hotfix-store-native')// ls只清空这两个键值
      ls.removeItem('hotfix-store-status')// ls只清空这两个键值
    },
    /*
     * @
     * @ 清除业务ss
     * @
     **/
    clearAppSS: function () {
      ss.clear()// ss直接清空
    },
    /*
     * @
     * @ 清除热更新存储信息
     * @ 出现意外时调用
     * @
     **/
    clearAll: function () {
      ls.removeItem('hotfix-store-native')// ls只清空这两个键值
      ls.removeItem('hotfix-store-status')// ls只清空这两个键值
      ss.clear()
    },
    /*
     * @
     * @ whole加载
     * @
     **/
    whole: function (paramwww) {
      this.syncLoadJavascript(paramwww, wholeList)
    },
    /*
     * @
     * @ pre加载
     * @
     **/
    pre: function (paramwww) {
      this.syncLoadJavascript(paramwww, preList)
    },
    /*
     * @
     * @ app加载
     * @
     **/
    app: function (paramwww) {
      this.syncLoadJavascript(paramwww, appList)
    },
    /*
     * @
     * @ 第一层
     * @ 如果已经有了加载session
     * @ 则证明热更新资源已经找到位置并迁移(iOS迁移)
     * @ 则加载热hot更新包的whole
     * @ 否则进入第二层
     * @
     **/
    isSessionWithLoad: function () {
      var wwwroot
      try {
        if (!SessStack) {
          throw '无SessStack信息'
        }
        if (+SessStack !== 1) {
          throw 'SessStack信息为0'
        }
        if (!SessPath) {
          throw '无SessPath信息'
        }
        wwwroot = SessPath + '/'
        this.clearAppSS()
        this.whole(wwwroot)
      } catch (e) {
        console.log(e)
        this.clearAppSS()
        this.isLocalWithLoad()// 进入第二层逻辑
      }
    },
    /*
     * @
     * @ 第二层
     * @ 先加载pre
     * @ 如果有localStorage
     * @ 则证明cache区有热更新资源待迁移(ios迁移)
     * @ 则埋上sess重新Load回第一层
     * @ 否则选择加载常规的app资源
     * @
     **/
    isLocalWithLoad: function () {
      var that = this
      var wwwroot = './'
      // pre加载
      this.pre(wwwroot)
      // 执行loader.load
      document.addEventListener('deviceready', function () {
        /*
          @ 判断isLocal
          @ 判断isLocal
          @ 判断isLocal
          @ 判断isLocal
        **/
        try {
          if ((!LocalStatus) || (!LocalNative)) {
            // 不可热更新
            // 只拉取沙盒区已有资源
            throw '无热更新存储信息'
          }
          if (!window.BuildInfo || !window.BuildInfo.version) {
            // 无BuildInfo信息
            // 只拉取沙盒区已有资源
            throw '有热更新存储-无BuildInfo信息'
          }

          var nativeVersion = window.BuildInfo.version
          // 查看状态关系以及版本关系是否可以开始热更新
          // LocalStatus === 1为可更新状态
          // nativeVersion === LocalNative为可更新状态
          if ((+LocalStatus !== 1) || (nativeVersion !== LocalNative)) {
            throw '有热更新资源-目前资源存在问题'
          }
          // 可以开始热更新
          console.log('可以开始热更新')
          window.HOTFIX.update(function (hotfixwww) {
            // 迁移hotfixwww已有资源
            // 更新session状态
            ss.setItem('hotfix-resume-stack', '1')
            ss.setItem('hotfix-resume-path', hotfixwww)
            location.reload()// 不用调用.app直接开始进入热更新版本
          })
        } catch (e) {
          console.log(e)
          wwwroot = './'
          that.clearAppLS()// 清空业务LS
          that.app(wwwroot)// 拉取app的js
        }
        /*
          @ 判断isLocal
          @ 判断isLocal
          @ 判断isLocal
          @ 判断isLocal
        **/
      })
    }
  }

  // 从第一层逻辑开始load
  loader.isSessionWithLoad()
})()
